@@ -1,7 +1,7 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
- * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
- * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2005-2011 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -45,7 +45,9 @@
 
 /**
  * thunar_dialogs_show_rename_file:
- * @parent : the parent #GtkWindow.
+ * @parent : a #GtkWidget on which the error dialog should be shown, or a #GdkScreen
+ *           if no #GtkWidget is known. May also be %NULL, in which case the default
+ *           #GdkScreen will be used.
  * @file   : the #ThunarFile we're going to rename.
  *
  * Displays the Thunar rename dialog for a single file rename.
@@ -54,7 +56,7 @@
  *               %NULL if there was no renaming required.
  **/
 ThunarJob *
-thunar_dialogs_show_rename_file (GtkWindow *parent,
+thunar_dialogs_show_rename_file (gpointer    parent,
                                  ThunarFile *file)
 {
   ThunarIconFactory *icon_factory;
@@ -67,21 +69,30 @@ thunar_dialogs_show_rename_file (GtkWindow *parent,
   GtkWidget         *label;
   GtkWidget         *image;
   GtkWidget         *table;
+  GtkWindow         *window;
   GdkPixbuf         *icon;
+  GdkScreen         *screen;
   glong              offset;
   gchar             *title;
   gint               response;
+  PangoLayout       *layout;
+  gint               layout_width;
+  gint               layout_offset;
+  gint               parent_width = 500;
 
-  _thunar_return_val_if_fail (GTK_IS_WINDOW (parent), FALSE);
+  _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WINDOW (parent), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
 
+  /* parse the parent window and screen */
+  screen = thunar_util_parse_parent (parent, &window);
+
   /* get the filename of the file */
   filename = thunar_file_get_display_name (file);
 
   /* create a new dialog window */
   title = g_strdup_printf (_("Rename \"%s\""), filename);
   dialog = gtk_dialog_new_with_buttons (title,
-                                        GTK_WINDOW (parent),
+                                        window,
                                         GTK_DIALOG_MODAL
                                         | GTK_DIALOG_NO_SEPARATOR
                                         | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -89,9 +100,12 @@ thunar_dialogs_show_rename_file (GtkWindow *parent,
                                         _("_Rename"), GTK_RESPONSE_OK,
                                         NULL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-  gtk_window_set_default_size (GTK_WINDOW (dialog), 300, -1);
   g_free (title);
 
+  /* move the dialog to the appropriate screen */
+  if (G_UNLIKELY (window == NULL && screen != NULL))
+    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+
   table = g_object_new (GTK_TYPE_TABLE, "border-width", 6, "column-spacing", 6, "row-spacing", 3, NULL);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), table, TRUE, TRUE, 0);
   gtk_widget_show (table);
@@ -123,8 +137,8 @@ thunar_dialogs_show_rename_file (GtkWindow *parent,
   /* check if we don't have a directory here */
   if (!thunar_file_is_directory (file))
     {
-      /* check if the filename contains a dot */
-      text = g_utf8_strrchr (filename, -1, '.');
+      /* check if the filename contains an extension */
+      text = thunar_util_str_get_extension (filename);
       if (G_LIKELY (text != NULL))
         {
           /* grab focus to the entry first, else the selection will be altered later */
@@ -139,6 +153,23 @@ thunar_dialogs_show_rename_file (GtkWindow *parent,
         }
     }
 
+  /* get the size the entry requires to render the full text */
+  layout = gtk_entry_get_layout (GTK_ENTRY (entry));
+  pango_layout_get_pixel_size (layout, &layout_width, NULL);
+  gtk_entry_get_layout_offsets (GTK_ENTRY (entry), &layout_offset, NULL);
+  layout_width += (layout_offset * 2) + (12 * 4) + 48; /* 12px free space in entry */
+
+  /* parent window width */
+  if (G_LIKELY (window != NULL))
+    {
+      /* keep below 90% of the parent window width */
+      gtk_window_get_size (GTK_WINDOW (window), &parent_width, NULL);
+      parent_width *= 0.90f;
+    }
+
+  /* resize the dialog to make long names fit as much as possible */
+  gtk_window_set_default_size (GTK_WINDOW (dialog), CLAMP (layout_width, 300, parent_width), -1);
+
   /* run the dialog */
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   if (G_LIKELY (response == GTK_RESPONSE_OK))
@@ -217,19 +248,14 @@ thunar_dialogs_show_about (GtkWindow   *parent,
   /* try to load the about logo */
   logo = gdk_pixbuf_new_from_file (DATADIR "/pixmaps/Thunar/Thunar-about-logo.png", NULL);
 
-#if !GTK_CHECK_VERSION (2, 18, 0)
-  gtk_about_dialog_set_email_hook (exo_gtk_url_about_dialog_hook, NULL, NULL);
-  gtk_about_dialog_set_url_hook (exo_gtk_url_about_dialog_hook, NULL, NULL);
-#endif
-
   /* open the about dialog */
   gtk_show_about_dialog (parent,
                          "artists", artists,
                          "authors", authors,
                          "comments", comments,
-                         "copyright", "Copyright \302\251 2004-2009 Benedikt Meurer\n"
-                                      "Copyright \302\251 2009-2010 Jannis Pohlmann\n"
-                                      "Copyright \302\251 2009-2010 Nick Schermer",
+                         "copyright", "Copyright \302\251 2004-2011 Benedikt Meurer\n"
+                                      "Copyright \302\251 2009-2011 Jannis Pohlmann\n"
+                                      "Copyright \302\251 2009-2012 Nick Schermer",
                          "destroy-with-parent", TRUE,
                          "documenters", documenters,
                          "license", XFCE_LICENSE_GPL,
@@ -277,6 +303,10 @@ thunar_dialogs_show_error (gpointer      parent,
 
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
 
+  /* do not display error dialog for already handled errors */
+  if (error && error->code == G_IO_ERROR_FAILED_HANDLED)
+    return;
+
   /* parse the parent pointer */
   screen = thunar_util_parse_parent (parent, &window);
 
@@ -311,67 +341,6 @@ thunar_dialogs_show_error (gpointer      parent,
 
 
 
-/**
- * thunar_dialogs_show_help:
- * @parent : a #GtkWidget on which the user manual should be shown, or a #GdkScreen
- *           if no #GtkWidget is known. May also be %NULL, in which case the default
- *           #GdkScreen will be used.
- * @page   : the name of the page of the user manual to display or %NULL to display
- *           the overview page.
- * @offset : the offset of the topic in the @page to display or %NULL to just display
- *           @page.
- *
- * Displays the Thunar user manual. If @page is not %NULL it specifies the basename
- * of the HTML file to display. @offset may refer to a anchor in the @page.
- **/
-void
-thunar_dialogs_show_help (gpointer     parent,
-                          const gchar *page,
-                          const gchar *offset)
-{
-  GdkScreen *screen;
-  GError    *error = NULL;
-  gchar     *command;
-  gchar     *tmp;
-
-  /* determine the screen on which to launch the help */
-  screen = thunar_util_parse_parent (parent, NULL);
-
-  /* generate the command for the documentation browser */
-  command = g_strdup (LIBDIR G_DIR_SEPARATOR_S "Thunar" G_DIR_SEPARATOR_S "ThunarHelp");
-
-  /* check if a page is given */
-  if (G_UNLIKELY (page != NULL))
-    {
-      /* append page as second parameter */
-      tmp = g_strconcat (command, " ", page, NULL);
-      g_free (command);
-      command = tmp;
-
-      /* check if an offset is given */
-      if (G_UNLIKELY (offset != NULL))
-        {
-          /* append offset as third parameter */
-          tmp = g_strconcat (command, " ", offset, NULL);
-          g_free (command);
-          command = tmp;
-        }
-    }
-
-  /* try to run the documentation browser */
-  if (!gdk_spawn_command_line_on_screen (screen, command, &error))
-    {
-      /* display an error message to the user */
-      thunar_dialogs_show_error (parent, error, _("Failed to open the documentation browser"));
-      g_error_free (error);
-    }
-
-  /* cleanup */
-  g_free (command);
-}
-
-
-
 /**
  * thunar_dialogs_show_job_ask:
  * @parent   : the parent #GtkWindow or %NULL.
@@ -395,7 +364,8 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
   GString     *secondary = g_string_sized_new (256);
   GString     *primary = g_string_sized_new (256);
   gint         response;
-  gint         n, m;
+  gint         n;
+  gboolean     has_cancel = FALSE;
 
   _thunar_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), THUNAR_JOB_RESPONSE_CANCEL);
   _thunar_return_val_if_fail (g_utf8_validate (question, -1, NULL), THUNAR_JOB_RESPONSE_CANCEL);
@@ -446,13 +416,10 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message), "%s", secondary->str);
 
   /* add the buttons based on the possible choices */
-  for (n = 5; n >= 0; --n)
+  for (n = 6; n >= 0; --n)
     {
-      /* "Cancel" should be the last button, but "Retry" was added last */
-      m = (n == 5) ? 3 : (n == 3) ? 5 : n;
-
       /* check if the response is set */
-      response = choices & (1 << m);
+      response = choices & (1 << n);
       if (response == 0)
         continue;
 
@@ -478,24 +445,37 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
           mnemonic = _("_Retry");
           break;
 
-        case THUNAR_JOB_RESPONSE_CANCEL:
-          response = GTK_RESPONSE_CANCEL;
-          mnemonic = _("_Cancel");
+        case THUNAR_JOB_RESPONSE_FORCE:
+          mnemonic = _("Copy _Anyway");
           break;
 
+        case THUNAR_JOB_RESPONSE_CANCEL:
+          /* cancel is always the last option */
+          has_cancel = TRUE;
+          continue;
+
         default:
           g_assert_not_reached ();
           break;
         }
 
       button = gtk_button_new_with_mnemonic (mnemonic);
-      GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+      gtk_widget_set_can_default (button, TRUE);
       gtk_dialog_add_action_widget (GTK_DIALOG (message), button, response);
       gtk_widget_show (button);
 
       gtk_dialog_set_default_response (GTK_DIALOG (message), response);
     }
 
+  if (has_cancel)
+    {
+      button = gtk_button_new_with_mnemonic (_("_Cancel"));
+      gtk_widget_set_can_default (button, TRUE);
+      gtk_dialog_add_action_widget (GTK_DIALOG (message), button, GTK_RESPONSE_CANCEL);
+      gtk_widget_show (button);
+      gtk_dialog_set_default_response (GTK_DIALOG (message), GTK_RESPONSE_CANCEL);
+    }
+
   /* run the question dialog */
   response = gtk_dialog_run (GTK_DIALOG (message));
   gtk_widget_destroy (message);
@@ -543,6 +523,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   gchar             *size_string;
   gchar             *text;
   gint               response;
+  gboolean           file_size_binary;
 
   _thunar_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), THUNAR_JOB_RESPONSE_CANCEL);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (src_file), THUNAR_JOB_RESPONSE_CANCEL);
@@ -551,6 +532,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   /* determine the style used to format dates */
   preferences = thunar_preferences_get ();
   g_object_get (G_OBJECT (preferences), "misc-date-style", &date_style, NULL);
+  g_object_get (G_OBJECT (preferences), "misc-file-size-binary", &file_size_binary, NULL);
   g_object_unref (G_OBJECT (preferences));
 
   /* setup the confirmation dialog */
@@ -560,6 +542,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
                                         | GTK_DIALOG_NO_SEPARATOR
                                         | GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                                        _("S_kip All"), THUNAR_JOB_RESPONSE_NO_ALL,
                                         _("_Skip"), THUNAR_JOB_RESPONSE_NO,
                                         _("Replace _All"), THUNAR_JOB_RESPONSE_YES_ALL,
                                         _("_Replace"), THUNAR_JOB_RESPONSE_YES,
@@ -568,6 +551,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
                                            THUNAR_JOB_RESPONSE_YES,
                                            THUNAR_JOB_RESPONSE_YES_ALL,
                                            THUNAR_JOB_RESPONSE_NO,
+                                           THUNAR_JOB_RESPONSE_NO_ALL,
                                            GTK_RESPONSE_CANCEL,
                                            -1);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_JOB_RESPONSE_YES);
@@ -635,7 +619,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_object_unref (G_OBJECT (icon));
   gtk_widget_show (image);
 
-  size_string = thunar_file_get_size_string (dst_file);
+  size_string = thunar_file_get_size_string_formatted (dst_file, file_size_binary);
   date_string = thunar_file_get_date_string (dst_file, THUNAR_FILE_DATE_MODIFIED, date_style);
   text = g_strdup_printf ("%s %s\n%s %s", _("Size:"), size_string, _("Modified:"), date_string);
   label = gtk_label_new (text);
@@ -666,7 +650,7 @@ thunar_dialogs_show_job_ask_replace (GtkWindow  *parent,
   g_object_unref (G_OBJECT (icon));
   gtk_widget_show (image);
 
-  size_string = thunar_file_get_size_string (src_file);
+  size_string = thunar_file_get_size_string_formatted (src_file, file_size_binary);
   date_string = thunar_file_get_date_string (src_file, THUNAR_FILE_DATE_MODIFIED, date_style);
   text = g_strdup_printf ("%s %s\n%s %s", _("Size:"), size_string, _("Modified:"), date_string);
   label = gtk_label_new (text);
@@ -753,3 +737,105 @@ thunar_dialogs_show_job_error (GtkWindow *parent,
   g_string_free (primary, TRUE);
 }
 
+
+
+gboolean
+thunar_dialogs_show_insecure_program (gpointer     parent,
+                                      const gchar *primary,
+                                      ThunarFile  *file,
+                                      const gchar *command)
+{
+  GdkScreen      *screen;
+  GtkWindow      *window;
+  gint            response;
+  GtkWidget      *dialog;
+  GString        *secondary;
+  ThunarFileMode  old_mode;
+  ThunarFileMode  new_mode;
+  GFileInfo      *info;
+  GError         *err = NULL;
+
+  _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
+  _thunar_return_val_if_fail (g_utf8_validate (command, -1, NULL), FALSE);
+
+  /* parse the parent window and screen */
+  screen = thunar_util_parse_parent (parent, &window);
+
+  /* secondary text */
+  secondary = g_string_new (NULL);
+  g_string_append_printf (secondary, _("The desktop file \"%s\" is in an insecure location "
+                                       "and not marked as executable. If you do not trust "
+                                       "this program, click Cancel."),
+                                       thunar_file_get_display_name (file));
+  g_string_append (secondary, "\n\n");
+  if (exo_str_looks_like_an_uri (command))
+    g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_URL"=%s", command);
+  else
+    g_string_append_printf (secondary, G_KEY_FILE_DESKTOP_KEY_EXEC"=%s", command);
+
+  /* allocate and display the error message dialog */
+  dialog = gtk_message_dialog_new (window,
+                                   GTK_DIALOG_MODAL |
+                                   GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_WARNING,
+                                   GTK_BUTTONS_NONE,
+                                   "%s", primary);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Launch Anyway"), GTK_RESPONSE_OK);
+  if (thunar_file_is_chmodable (file))
+    gtk_dialog_add_button (GTK_DIALOG (dialog), _("Mark _Executable"), GTK_RESPONSE_APPLY);
+  gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
+  if (screen != NULL && window == NULL)
+    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+  gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary->str);
+  g_string_free (secondary, TRUE);
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+
+  /* check if we should make the file executable */
+  if (response == GTK_RESPONSE_APPLY)
+    {
+      /* try to query information about the file */
+      info = g_file_query_info (thunar_file_get_file (file),
+                                G_FILE_ATTRIBUTE_UNIX_MODE,
+                                G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                NULL, &err);
+
+      if (G_LIKELY (info != NULL))
+        {
+          if (g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_UNIX_MODE))
+            {
+              /* determine the current mode */
+              old_mode = g_file_info_get_attribute_uint32 (info, G_FILE_ATTRIBUTE_UNIX_MODE);
+
+              /* generate the new mode */
+              new_mode = old_mode | THUNAR_FILE_MODE_USR_EXEC | THUNAR_FILE_MODE_GRP_EXEC | THUNAR_FILE_MODE_OTH_EXEC;
+
+              if (old_mode != new_mode)
+                {
+                  g_file_set_attribute_uint32 (thunar_file_get_file (file),
+                                               G_FILE_ATTRIBUTE_UNIX_MODE, new_mode,
+                                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+                                               NULL, &err);
+                }
+            }
+          else
+            {
+              g_warning ("No %s attribute found", G_FILE_ATTRIBUTE_UNIX_MODE);
+            }
+
+          g_object_unref (info);
+        }
+
+      if (err != NULL)
+        {
+          thunar_dialogs_show_error (parent, err, ("Unable to mark launcher executable"));
+          g_error_free (err);
+        }
+
+      /* just launch */
+      response = GTK_RESPONSE_OK;
+    }
+
+  return (response == GTK_RESPONSE_OK);
+}
